@@ -4,11 +4,13 @@ import pandas as pd
 from nltk.corpus import stopwords
 from nltk import download
 from gensim.models import KeyedVectors
+import gensim.downloader as api
 from sentence_transformers import SentenceTransformer, util, InputExample, losses, models
 from string import punctuation
 from nltk.stem import PorterStemmer
 from nltk.stem.isri import ISRIStemmer
 
+
 def preprocess(sentence, remove_punct, remove_stop_words, stemm, lang='en'):
     if lang.lower() == 'en':
         ps = PorterStemmer()
@@ -48,6 +50,8 @@ def preprocess(sentence, remove_punct, remove_stop_words, stemm, lang='en'):
         
 class sentence_tranformer():
     def __init__(self, source_names, target_names, model=None):
+        if not source_names:
+            raise Exception('Inputs are empty')
         self.source_names = source_names
         self.target_names = [target  for target in target_names if not (pd.isnull(target))] 
         self.model = model
@@ -56,6 +60,7 @@ class sentence_tranformer():
             print('initializing the model...')
             self.model = SentenceTransformer('paraphrase-MiniLM-L6-v2')
 
+        # encode the targets
         self.encoded_targets = self.model.encode(self.target_names)
         
 
@@ -126,14 +131,17 @@ class sentence_tranformer():
 
 
 class word_mover_distance():
-    def __init__(self, source_names, target_names, model):
+    def __init__(self, source_names, target_names, model, return_match_idx=False):
+        if not source_names:
+            raise Exception('Inputs are empty')
         self.source_names = source_names
         self.target_names = target_names
         self.model = model
-        
+        self.return_match_idx = return_match_idx
         # if no model is provided use the default model
-        # if model is None:
-        #     self.model = KeyedVectors.load_word2vec_format('BioWordVec_PubMed_MIMICIII_d200.vec.bin', binary=True)
+        if model is None:
+            print('initializing the model (English model)...')
+            self.model = api.load('glove-wiki-gigaword-300')
 
     def match(self, topn=1):
         '''
@@ -158,53 +166,78 @@ class word_mover_distance():
         
         # get topn results
         wmd_sorted = np.sort(wmd_results)
-        results = []
         scores = []
+        indexes = []
         for x in wmd_sorted:
-            if len(results) == self.topn:
+            if len(indexes) == self.topn:
                 break
             # convert distance to score
             scores.append(1 - x)
             for y in np.where(wmd_results == x):
-                results.append(y[0])
-                if len(results) == self.topn:
+                indexes.append(y[0])
+                if len(indexes) == self.topn:
                     break    
-        targets = [self.target_names[idx] for idx in results]
+        targets = [self.target_names[idx] for idx in indexes]
         
         # fill empty topn results 
         while len(targets) < self.topn:
+            indexes.append(None)
             targets.append(None)
             scores.append(None)
 
-        return targets, scores
+        return targets, scores, indexes
     
 
     def top_wmd_distance(self):
         results = np.array([self.min_wmd_distance(input) for input in self.source_names])
         self.targets = results[:, 0]
         self.top_scores = results[:, 1]
+        self.match_idxs = results[:, 2]
 
 
     def _make_matchdf(self):
         ''' Build dataframe for result return '''
+        if not self.return_match_idx:
+            match_list = []
+            for source, targets, top_scores in zip(self.source_names, self.targets, self.top_scores):
+                row = []
+                row.append(source)
+                if targets is not None:
+                    # loop over results of multi matches
+                    for target, top_score in zip(targets, top_scores):
+                        row.append(target)
+                        row.append(top_score) 
+                match_list.append(tuple(row))
 
-        match_list = []
-        for source, target, top_score in zip(self.source_names, self.targets, self.top_scores):
-            row = []
-            row.append(source)
-            if target is not None:
-                for x, y in zip(target, top_score):
-                    row.append(x)
-                    row.append(y) 
-            match_list.append(tuple(row))
+            # prepare columns names
+            colnames = ['source', 'prediction', 'score']
+            
+            for i in range(2, self.topn+1):
+                colnames.append(f'prediction_{i}')
+                colnames.append(f'score_{i}')
 
-        # prepare columns names
-        colnames = ['source', 'prediction', 'score']
-        
-        for i in range(2, self.topn+1):
-            colnames.append(f'prediction_{i}')
-            colnames.append(f'score_{i}')
+            match_df = pd.DataFrame(match_list, columns=colnames)
+        else:
+            match_list = []
+            for source, targets, top_scores, match_idxs in zip(self.source_names, self.targets, self.top_scores, self.match_idxs):
+                row = []
+                row.append(source)
+                if targets is not None:
+                    # loop over results of multi matches
+                    for target, top_score, match_idx in zip(targets, top_scores, match_idxs):
+                        row.append(target)
+                        row.append(top_score) 
+                        row.append(match_idx)
+                match_list.append(tuple(row))
 
-        match_df = pd.DataFrame(match_list, columns=colnames)
+            # prepare columns names
+            colnames = ['source', 'prediction', 'score', 'match_idx']
+            
+            for i in range(2, self.topn+1):
+                colnames.append(f'prediction_{i}')
+                colnames.append(f'score_{i}')
+                colnames.append(f'match_idx_{i}')
+
+            match_df = pd.DataFrame(match_list, columns=colnames)  
         
         return match_df
\ No newline at end of file
